using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class LevelGenerator : MonoBehaviour {
	[Range(0,1)]
	public float densityStart = 0.2f;	// overall density

	[Range(0,1)]
	public float densityEnd = 0.5f;	// overall density

	public float densityLerpDuration = 60.0f;

	[Range(1,10)]
	public int maxPerRow;

	[Range(0,1)]
	public float[] probability;	// individual probability for each type (0...1)


	private UnitMapper unitMapper;


	// Use this for initialization
	void Start () {
		print("LevelGenerator.Start");
		unitMapper = GetComponent<UnitMapper>();
		normalizeProbabilities();
	}

	void normalizeProbabilities() {
		// normalize probabilities
		float totalProbability = 0.0f;
		for(int i=0; i<probability.Length; i++) {
			totalProbability += probability[i];
		}
//		print ("totalProbability: " + totalProbability);
		if(totalProbability > 0.0f) {
			float probScaler = 1.0f/totalProbability;
			for(int i=0; i<probability.Length; i++) {
				probability[i] *= probScaler;
			}
		}
	}


	// receive a tokenGenerator (which has list of Keys) and generate random level data
	public string[] generate(List<char> keys, int length, float time) {
		normalizeProbabilities();

		string[] ret = new string[length];

		float density;
		if(densityLerpDuration > 0) {
			float timeRatio = Mathf.Clamp01(time / densityLerpDuration);
			density = Mathf.Lerp(densityStart, densityEnd, timeRatio);
		} else {
			density = densityStart;
		}
//		print(time + " : " + density);

		// iterate rows
		for(int i=0; i<length; i++) {

			// default row to all .
			ret[i] = new string('.', (int)unitMapper.cellCount.x);

			int iterations = Mathf.FloorToInt(Random.Range (1.0f, (float)maxPerRow));
			for(int n=0; n<iterations; n++) {

				// pick random object
				int objectIndex = Mathf.FloorToInt(Random.Range(0.0f, (float)keys.Count));

				// check number against probability of object
				if(Random.Range (0.0f, 1.0f) < probability[objectIndex] * density) {
					// update char at charIndex
					int charIndex = Mathf.FloorToInt(Random.Range (0.0f, (float)unitMapper.cellCount.x));
					ret[i] = ret[i].Remove(charIndex, 1);
					ret[i] = ret[i].Insert(charIndex, new string(keys[objectIndex], 1));
				}
			}
		}

		return ret;
	}
	
	// Update is called once per frame
	void Update () {
	
	}
}
