using UnityEngine;
using System.Collections;

public class LevelManager : MonoBehaviour {
	public string levelName;
	public float maxNoise;
	public float maxSpeed = 0;
	public float slowDown = 0.5f;
	public float acceleration = 1;
	public float speed = 0;	// current speed
	public float position = 0;	// position of screen window in relation to data file, in world units
	public bool isAlive;
	public float timeBeforeDeath = 2.0f;
	public Texture gameOverImage;

	//----------------------------------------
	private string[] levelData;	// contains text file with cell data
	private UnitMapper unitMapper;
	private TokenGenerator tokenGenerator;
	private int positionCell;	// position of screen window in relation to data file, in cells
	private int oldPositionCell; // previous frame of above

	private float deathTimer = 0.0f;

	private GameObject timeClock;
	private TimeClock timeClockComponent;
	private float timePlayed = 0.0f;

	//----------------------------------------
	void loadDataFile(string filepath) {
		print ("loadDataFile: " + filepath);
//		levelData = System.IO.File.ReadAllText(filepath).ToUpper().Split();
//		print (levelData);
	}
	
	public void reset() {
		Application.LoadLevel(Application.loadedLevel);
	}

	public void hitSlowdown() {
		speed *= slowDown;
	}

	public void hitKill() {
		speed = 0;
		deathTimer = 0.0f;
		if (isAlive) 
		{
			isAlive = false;

			// reset the font size 
			timeClockComponent.largeFont.fontSize = 95;

			Destroy (timeClock);
		}
	}

	// Use this for initialization
	void Start () {
		print ("LevelManager.Start");
		isAlive = true;
		deathTimer = 0.0f;

		tokenGenerator = GetComponent<TokenGenerator>();
		unitMapper = GetComponent<UnitMapper>();
		timeClock = GameObject.Find ("Timer");
		timeClockComponent = timeClock.GetComponent<TimeClock> ();

		// load data text file
		print(levelName);
		if(levelName != "") loadDataFile ("assets/LevelManager/levels/" + levelName);
//		else levelData = GetComponent<LevelGenerator>().generate(tokenGenerator);

//		tokenGenerator.generate(levelData, 0, levelData.Length, unitMapper, 0);
	}

	// Update is called once per frame
	void Update () {
		// save time played
		timePlayed = timeClockComponent.timePlayed; 


		// check if new cell
		positionCell = Mathf.CeilToInt(unitMapper.worldToCellPoint(new Vector3(0, -position, 0)).y);
		if(positionCell != oldPositionCell) {
			oldPositionCell = positionCell;

			// generate level data
			levelData = GetComponent<LevelGenerator>().generate(tokenGenerator.keys, 1, timePlayed);

			// generate tokens
			tokenGenerator.generate(levelData, 0, levelData.Length, unitMapper, -unitMapper.screenSize.y * 1.4f);
//			print ("NEW POSITION CELL" + positionCell + " : " + levelData);
		}


		// shake screen
		if(maxSpeed > 0) {
			float r = speed / maxSpeed;
			r *= r;
			Camera.main.transform.position = Random.insideUnitSphere * maxNoise * r - new Vector3(0, 0, 100);
		}
	}

	void OnGUI()
	{
		if(isAlive==false)
		{
			//deathTimer += Time.deltaTime;
			if(true) //deathTimer >= timeBeforeDeath)
			{
				//GUI.DrawTexture (new Rect (Screen.width/2-gameOverImage.height/2, Screen.height/2-gameOverImage.height/2, gameOverImage.width, gameOverImage.height), gameOverImage);

				GUIStyle centeredStyle = GUI.skin.GetStyle("Label"); //GUI.skin.GetStyle("Label");
				centeredStyle.alignment = TextAnchor.MiddleCenter;

				centeredStyle.fontSize = (int)(((float)Screen.width/1920.0f)*60.0f);
				centeredStyle.font = timeClockComponent.largeFont.font;
				centeredStyle.normal.textColor = Color.white;

				string endstr = "";
				if( timePlayed > 5.0)
					endstr+="WELL DONE!\n";
				endstr += "YOU LASTED " + timePlayed.ToString("f1") + " SECONDS\n"; 
				endstr += "CLICK TO PLAY AGAIN";

				GUI.Label(new Rect(0, 0, Screen.width, Screen.height), endstr, centeredStyle );

				if (Input.GetKeyDown ("space") || Input.GetMouseButtonDown(0)) {
					Application.LoadLevel(0);
				}
			}
		}
	}

	// physics update
	void FixedUpdate() {
//		if(isAlive == false) return;
		if (maxSpeed == 0 || speed < maxSpeed) {
			speed += acceleration;// * Time.deltaTime;
		} else {
			speed = maxSpeed;
		}
		position += speed;// * Time.deltaTime;
		transform.position = new Vector3(0, position, 0);
	}

}
